@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2015 Drew Noakes
+ * Copyright 2002-2017 Drew Noakes
  *
  *    Licensed under the Apache License, Version 2.0 (the "License");
  *    you may not use this file except in compliance with the License.
@@ -66,21 +66,28 @@ public class GeoTagMapBuilder
             }
         });
 
+        if (files == null)
+        {
+            System.err.println("No matching files found.");
+            System.exit(1);
+        }
+
         Collection<PhotoLocation> photoLocations = new ArrayList<PhotoLocation>();
         for (File file : files)
         {
             // Read all metadata from the image
             Metadata metadata = ImageMetadataReader.readMetadata(file);
             // See whether it has GPS data
-            GpsDirectory gpsDirectory = metadata.getDirectory(GpsDirectory.class);
-            if (gpsDirectory == null)
-                continue;
-            // Try to read out the location, making sure it's non-zero
-            GeoLocation geoLocation = gpsDirectory.getGeoLocation();
-            if (geoLocation == null || geoLocation.isZero())
-                continue;
-            // Add to our collection for use below
-            photoLocations.add(new PhotoLocation(geoLocation, file));
+            Collection<GpsDirectory> gpsDirectories = metadata.getDirectoriesOfType(GpsDirectory.class);
+            for (GpsDirectory gpsDirectory : gpsDirectories) {
+                // Try to read out the location, making sure it's non-zero
+                GeoLocation geoLocation = gpsDirectory.getGeoLocation();
+                if (geoLocation != null && !geoLocation.isZero()) {
+                    // Add to our collection for use below
+                    photoLocations.add(new PhotoLocation(geoLocation, file));
+                    break;
+                }
+            }
         }
 
         // Write output to the console.
