@@ -25,6 +25,7 @@ import com.drew.lang.StreamReader;
 import com.drew.lang.annotations.NotNull;
 import com.drew.metadata.Metadata;
 import com.drew.metadata.bmp.BmpReader;
+import com.drew.metadata.file.FileSystemMetadataReader;
 
 import java.io.*;
 
@@ -38,15 +39,15 @@ public class BmpMetadataReader
     @NotNull
     public static Metadata readMetadata(@NotNull File file) throws IOException
     {
-        FileInputStream stream = null;
+        InputStream inputStream = new FileInputStream(file);
+        Metadata metadata;
         try {
-            stream = new FileInputStream(file);
-            return readMetadata(stream);
+            metadata = readMetadata(inputStream);
         } finally {
-            if (stream != null) {
-                stream.close();
-            }
+            inputStream.close();
         }
+        new FileSystemMetadataReader().read(file, metadata);
+        return metadata;
     }
 
     @NotNull
