@@ -35,7 +35,7 @@ public class ByteTrie<T>
     /** A node in the trie. Has children and may have an associated value. */
     static class ByteTrieNode<T>
     {
-        private final Map<Byte,ByteTrieNode<T>> _children = new HashMap<Byte, ByteTrieNode<T>>();
+        private final Map<Byte, ByteTrieNode<T>> _children = new HashMap<Byte, ByteTrieNode<T>>();
         private T _value = null;
 
         public void setValue(T value)
@@ -86,6 +86,8 @@ public class ByteTrie<T>
                 depth++;
             }
         }
+        if (depth == 0)
+            throw new IllegalArgumentException("Parts must contain at least one byte.");
         node.setValue(value);
         _maxDepth = Math.max(_maxDepth, depth);
     }
