@@ -26,10 +26,7 @@ import com.drew.lang.SequentialReader;
 import com.drew.lang.annotations.NotNull;
 import com.drew.lang.annotations.Nullable;
 
-import com.drew.metadata.Directory;
-import com.drew.metadata.ErrorDirectory;
-import com.drew.metadata.Metadata;
-import com.drew.metadata.StringValue;
+import com.drew.metadata.*;
 import com.drew.metadata.gif.GifControlDirectory.DisposalMethod;
 import com.drew.metadata.icc.IccReader;
 import com.drew.metadata.xmp.XmpReader;
@@ -72,8 +69,17 @@ public class GifReader
             return;
 
         try {
-            // Skip over any global colour table
-            Integer globalColorTableSize = header.getInteger(GifHeaderDirectory.TAG_COLOR_TABLE_SIZE);
+            // Skip over any global colour table if GlobalColorTable is present.
+            Integer globalColorTableSize = null;
+            try {
+                boolean hasGlobalColorTable = header.getBoolean(GifHeaderDirectory.TAG_HAS_GLOBAL_COLOR_TABLE);
+                if(hasGlobalColorTable) {
+                    globalColorTableSize = header.getInteger(GifHeaderDirectory.TAG_COLOR_TABLE_SIZE);
+                }
+            } catch (MetadataException e) {
+                // This exception should never occur here.
+                metadata.addDirectory(new ErrorDirectory("GIF did not had hasGlobalColorTable bit."));
+            }
             if (globalColorTableSize != null)
             {
                 // Colour table has R/G/B byte triplets
@@ -115,6 +121,7 @@ public class GifReader
                         // Anything other than these types is unexpected.
                         // GIF87a spec says to keep reading until a separator is found.
                         // GIF89a spec says file is corrupt.
+                        metadata.addDirectory(new ErrorDirectory("Unknown gif block marker found."));
                         return;
                     }
                 }
@@ -172,7 +179,7 @@ public class GifReader
         // First three bits = (BPP - 1)
         int colorTableSize = 1 << ((flags & 7) + 1);
         int bitsPerPixel = ((flags & 0x70) >> 4) + 1;
-        boolean hasGlobalColorTable = (flags & 0xf) != 0;
+        boolean hasGlobalColorTable = (flags >> 7) != 0;
 
         headerDirectory.setInt(GifHeaderDirectory.TAG_COLOR_TABLE_SIZE, colorTableSize);
 
@@ -270,7 +277,7 @@ public class GifReader
         else if (extensionType.equals("ICCRGBG1012"))
         {
             // ICC profile extension
-            byte[] iccBytes = gatherBytes(reader, reader.getByte());
+            byte[] iccBytes = gatherBytes(reader, ((int) reader.getByte()) & 0xff);
             if (iccBytes.length != 0)
                 new IccReader().extract(new ByteArrayReader(iccBytes), metadata);
         }
@@ -301,8 +308,8 @@ public class GifReader
 
         short packedFields = reader.getUInt8();
         directory.setObject(GifControlDirectory.TAG_DISPOSAL_METHOD, DisposalMethod.typeOf((packedFields >> 2) & 7));
-        directory.setBoolean(GifControlDirectory.TAG_USER_INPUT_FLAG, (packedFields & 2) >> 1 == 1 ? true : false);
-        directory.setBoolean(GifControlDirectory.TAG_TRANSPARENT_COLOR_FLAG, (packedFields & 1) == 1 ? true : false);
+        directory.setBoolean(GifControlDirectory.TAG_USER_INPUT_FLAG, (packedFields & 2) >> 1 == 1);
+        directory.setBoolean(GifControlDirectory.TAG_TRANSPARENT_COLOR_FLAG, (packedFields & 1) == 1);
         directory.setInt(GifControlDirectory.TAG_DELAY, reader.getUInt16());
         directory.setInt(GifControlDirectory.TAG_TRANSPARENT_COLOR_INDEX, reader.getUInt8());
 
@@ -322,15 +329,15 @@ public class GifReader
         imageDirectory.setInt(GifImageDirectory.TAG_HEIGHT, reader.getUInt16());
 
         byte flags = reader.getByte();
-        boolean hasColorTable = (flags & 0x7) != 0;
+        boolean hasColorTable = (flags >> 7) != 0;
         boolean isInterlaced = (flags & 0x40) != 0;
-        boolean isColorTableSorted = (flags & 0x20) != 0;
 
         imageDirectory.setBoolean(GifImageDirectory.TAG_HAS_LOCAL_COLOUR_TABLE, hasColorTable);
         imageDirectory.setBoolean(GifImageDirectory.TAG_IS_INTERLACED, isInterlaced);
 
         if (hasColorTable)
         {
+            boolean isColorTableSorted = (flags & 0x20) != 0;
             imageDirectory.setBoolean(GifImageDirectory.TAG_IS_COLOR_TABLE_SORTED, isColorTableSorted);
 
             int bitsPerPixel = (flags & 0x7) + 1;
@@ -375,7 +382,7 @@ public class GifReader
         {
             buffer.write(reader.getBytes(length), 0, length);
 
-            length = reader.getByte();
+            length = reader.getByte() & 0xff;
         }
 
         return buffer.toByteArray();
