@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2015 Drew Noakes
+ * Copyright 2002-2017 Drew Noakes
  *
  *    Licensed under the Apache License, Version 2.0 (the "License");
  *    you may not use this file except in compliance with the License.
@@ -23,58 +23,89 @@ package com.drew.metadata.icc;
 import com.drew.imaging.jpeg.JpegSegmentMetadataReader;
 import com.drew.imaging.jpeg.JpegSegmentType;
 import com.drew.lang.ByteArrayReader;
+import com.drew.lang.DateUtil;
 import com.drew.lang.RandomAccessReader;
 import com.drew.lang.annotations.NotNull;
+import com.drew.lang.annotations.Nullable;
 import com.drew.metadata.Directory;
 import com.drew.metadata.Metadata;
 import com.drew.metadata.MetadataReader;
 
 import java.io.IOException;
-import java.util.Arrays;
-import java.util.Calendar;
-import java.util.Date;
-import java.util.TimeZone;
+import java.util.Collections;
 
 /**
  * Reads an ICC profile.
+ * <p>
+ * More information about ICC:
  * <ul>
  * <li>http://en.wikipedia.org/wiki/ICC_profile</li>
  * <li>http://www.sno.phy.queensu.ca/~phil/exiftool/TagNames/ICC_Profile.html</li>
+ * <li>https://developer.apple.com/library/mac/samplecode/ImageApp/Listings/ICC_h.html</li>
  * </ul>
  *
  * @author Yuri Binev
- * @author Drew Noakes
+ * @author Drew Noakes https://drewnoakes.com
  */
 public class IccReader implements JpegSegmentMetadataReader, MetadataReader
 {
+    public static final String JPEG_SEGMENT_PREAMBLE = "ICC_PROFILE";
+
     @NotNull
     public Iterable<JpegSegmentType> getSegmentTypes()
     {
-        return Arrays.asList(JpegSegmentType.APP2);
+        return Collections.singletonList(JpegSegmentType.APP2);
     }
 
-    public boolean canProcess(@NotNull byte[] segmentBytes, @NotNull JpegSegmentType segmentType)
+    public void readJpegSegments(@NotNull Iterable<byte[]> segments, @NotNull Metadata metadata, @NotNull JpegSegmentType segmentType)
     {
-        return segmentBytes.length > 10 && "ICC_PROFILE".equalsIgnoreCase(new String(segmentBytes, 0, 11));
-    }
+        final int preambleLength = JPEG_SEGMENT_PREAMBLE.length();
 
-    public void extract(@NotNull byte[] segmentBytes, @NotNull Metadata metadata, @NotNull JpegSegmentType segmentType)
-    {
-        // skip the first 14 bytes
-        byte[] iccProfileBytes = new byte[segmentBytes.length - 14];
-        System.arraycopy(segmentBytes, 14, iccProfileBytes, 0, segmentBytes.length - 14);
+        // ICC data can be spread across multiple JPEG segments.
+        // We concat them together in this buffer for later processing.
+        byte[] buffer = null;
 
-        extract(new ByteArrayReader(iccProfileBytes), metadata);
+        for (byte[] segmentBytes : segments) {
+            // Skip any segments that do not contain the required preamble
+            if (segmentBytes.length < preambleLength || !JPEG_SEGMENT_PREAMBLE.equalsIgnoreCase(new String(segmentBytes, 0, preambleLength)))
+                continue;
+
+            // NOTE we ignore three bytes here -- are they useful for anything?
+
+            // Grow the buffer
+            if (buffer == null) {
+                buffer = new byte[segmentBytes.length - 14];
+                // skip the first 14 bytes
+                System.arraycopy(segmentBytes, 14, buffer, 0, segmentBytes.length - 14);
+            } else {
+                byte[] newBuffer = new byte[buffer.length + segmentBytes.length - 14];
+                System.arraycopy(buffer, 0, newBuffer, 0, buffer.length);
+                System.arraycopy(segmentBytes, 14, newBuffer, buffer.length, segmentBytes.length - 14);
+                buffer = newBuffer;
+            }
+        }
+
+        if (buffer != null)
+            extract(new ByteArrayReader(buffer), metadata);
     }
 
     public void extract(@NotNull final RandomAccessReader reader, @NotNull final Metadata metadata)
     {
-        // TODO review whether the 'tagPtr' values below really do require ICC processing to work with a RandomAccessReader
+        extract(reader, metadata, null);
+    }
 
-        final IccDirectory directory = metadata.getOrCreateDirectory(IccDirectory.class);
+    public void extract(@NotNull final RandomAccessReader reader, @NotNull final Metadata metadata, @Nullable Directory parentDirectory)
+    {
+        // TODO review whether the 'tagPtr' values below really do require RandomAccessReader or whether SequentialReader may be used instead
+
+        IccDirectory directory = new IccDirectory();
+
+        if (parentDirectory != null)
+            directory.setParent(parentDirectory);
 
         try {
-            directory.setInt(IccDirectory.TAG_PROFILE_BYTE_COUNT, reader.getInt32(IccDirectory.TAG_PROFILE_BYTE_COUNT));
+            int profileByteCount = reader.getInt32(IccDirectory.TAG_PROFILE_BYTE_COUNT);
+            directory.setInt(IccDirectory.TAG_PROFILE_BYTE_COUNT, profileByteCount);
 
             // For these tags, the int value of the tag is in fact it's offset within the buffer.
             set4ByteString(directory, IccDirectory.TAG_CMM_TYPE, reader);
@@ -122,6 +153,8 @@ public class IccReader implements JpegSegmentMetadataReader, MetadataReader
         } catch (IOException ex) {
             directory.addError("Exception reading ICC profile: " + ex.getMessage());
         }
+
+        metadata.addDirectory(directory);
     }
 
     private void set4ByteString(@NotNull Directory directory, int tagType, @NotNull RandomAccessReader reader) throws IOException
@@ -156,12 +189,17 @@ public class IccReader implements JpegSegmentMetadataReader, MetadataReader
         final int M = reader.getUInt16(tagType + 8);
         final int s = reader.getUInt16(tagType + 10);
 
-//        final Date value = new Date(Date.UTC(y - 1900, m - 1, d, h, M, s));
-        final Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
-        calendar.set(y, m, d, h, M, s);
-        final Date value = calendar.getTime();
-
-        directory.setDate(tagType, value);
+        if (DateUtil.isValidDate(y, m - 1, d) && DateUtil.isValidTime(h, M, s))
+        {
+            String dateString = String.format("%04d:%02d:%02d %02d:%02d:%02d", y, m, d, h, M, s);
+            directory.setString(tagType, dateString);
+        }
+        else
+        {
+            directory.addError(String.format(
+                "ICC data describes an invalid date/time: year=%d month=%d day=%d hour=%d minute=%d second=%d",
+                y, m, d, h, M, s));
+        }
     }
 
     @NotNull
