@@ -38,7 +38,7 @@ import com.drew.metadata.Tag;
 import com.drew.metadata.exif.ExifIFD0Directory;
 import com.drew.metadata.exif.ExifSubIFDDirectory;
 import com.drew.metadata.exif.ExifThumbnailDirectory;
-import com.drew.metadata.file.FileMetadataDirectory;
+import com.drew.metadata.file.FileSystemDirectory;
 import com.drew.metadata.xmp.XmpDirectory;
 
 import java.io.*;
@@ -170,11 +170,14 @@ public class ProcessAllImagesInFolderUtility
 
     abstract static class FileHandlerBase implements FileHandler
     {
+        // TODO obtain these from FileType enum directly
         private final Set<String> _supportedExtensions = new HashSet<String>(
             Arrays.asList(
                 "jpg", "jpeg", "png", "gif", "bmp", "ico", "webp", "pcx", "ai", "eps",
                 "nef", "crw", "cr2", "orf", "arw", "raf", "srw", "x3f", "rw2", "rwl",
-                "tif", "tiff", "psd", "dng"));
+                "tif", "tiff", "psd", "dng",
+                "3g2", "3gp", "m4v", "mov", "mp4",
+                "pbm", "pnm", "pgm"));
 
         private int _processedFileCount = 0;
         private int _exceptionCount = 0;
@@ -321,7 +324,7 @@ public class ProcessAllImagesInFolderUtility
                             if (description == null)
                                 description = "";
                             // Skip the file write-time as this changes based on the time at which the regression test image repository was cloned
-                            if (directory instanceof FileMetadataDirectory && tag.getTagType() == FileMetadataDirectory.TAG_FILE_MODIFIED_DATE)
+                            if (directory instanceof FileSystemDirectory && tag.getTagType() == FileSystemDirectory.TAG_FILE_MODIFIED_DATE)
                                 description = "<omitted for regression testing as checkout dependent>";
                             writer.format("[%s - %s] %s = %s%s", directoryName, tag.getTagTypeHex(), tagName, description, NEW_LINE);
                         }
