@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2015 Drew Noakes
+ * Copyright 2002-2017 Drew Noakes
  *
  *    Licensed under the Apache License, Version 2.0 (the "License");
  *    you may not use this file except in compliance with the License.
@@ -36,6 +36,7 @@ import static org.junit.Assert.*;
  *
  * @author Drew Noakes https://drewnoakes.com
  */
+@SuppressWarnings("ConstantConditions")
 public class IptcReaderTest
 {
     @NotNull
@@ -44,7 +45,7 @@ public class IptcReaderTest
         Metadata metadata = new Metadata();
         byte[] bytes = FileUtil.readBytes(filePath);
         new IptcReader().extract(new SequentialByteArrayReader(bytes), metadata, bytes.length);
-        IptcDirectory directory = metadata.getDirectory(IptcDirectory.class);
+        IptcDirectory directory = metadata.getFirstDirectoryOfType(IptcDirectory.class);
         assertNotNull(directory);
         return directory;
     }
@@ -63,49 +64,49 @@ public class IptcReaderTest
         assertArrayEquals(new String[] { "Supl. Category2", "Supl. Category1", "Cat" }, directory.getStringArray(tags[0].getTagType()));
 
         assertEquals(IptcDirectory.TAG_COPYRIGHT_NOTICE, tags[1].getTagType());
-        assertEquals("Copyright", directory.getObject(tags[1].getTagType()));
+        assertEquals("Copyright", directory.getString(tags[1].getTagType()));
 
         assertEquals(IptcDirectory.TAG_SPECIAL_INSTRUCTIONS, tags[2].getTagType());
-        assertEquals("Special Instr.", directory.getObject(tags[2].getTagType()));
+        assertEquals("Special Instr.", directory.getString(tags[2].getTagType()));
 
         assertEquals(IptcDirectory.TAG_HEADLINE, tags[3].getTagType());
-        assertEquals("Headline", directory.getObject(tags[3].getTagType()));
+        assertEquals("Headline", directory.getString(tags[3].getTagType()));
 
         assertEquals(IptcDirectory.TAG_CAPTION_WRITER, tags[4].getTagType());
-        assertEquals("CaptionWriter", directory.getObject(tags[4].getTagType()));
+        assertEquals("CaptionWriter", directory.getString(tags[4].getTagType()));
 
         assertEquals(IptcDirectory.TAG_CAPTION, tags[5].getTagType());
-        assertEquals("Caption", directory.getObject(tags[5].getTagType()));
+        assertEquals("Caption", directory.getString(tags[5].getTagType()));
 
         assertEquals(IptcDirectory.TAG_ORIGINAL_TRANSMISSION_REFERENCE, tags[6].getTagType());
-        assertEquals("Transmission", directory.getObject(tags[6].getTagType()));
+        assertEquals("Transmission", directory.getString(tags[6].getTagType()));
 
         assertEquals(IptcDirectory.TAG_COUNTRY_OR_PRIMARY_LOCATION_NAME, tags[7].getTagType());
-        assertEquals("Country", directory.getObject(tags[7].getTagType()));
+        assertEquals("Country", directory.getString(tags[7].getTagType()));
 
         assertEquals(IptcDirectory.TAG_PROVINCE_OR_STATE, tags[8].getTagType());
-        assertEquals("State", directory.getObject(tags[8].getTagType()));
+        assertEquals("State", directory.getString(tags[8].getTagType()));
 
         assertEquals(IptcDirectory.TAG_CITY, tags[9].getTagType());
-        assertEquals("City", directory.getObject(tags[9].getTagType()));
+        assertEquals("City", directory.getString(tags[9].getTagType()));
 
         assertEquals(IptcDirectory.TAG_DATE_CREATED, tags[10].getTagType());
-        assertEquals(new java.util.GregorianCalendar(2000, 0, 1).getTime(), directory.getObject(tags[10].getTagType()));
+        assertEquals("20000101", directory.getString(tags[10].getTagType()));
 
         assertEquals(IptcDirectory.TAG_OBJECT_NAME, tags[11].getTagType());
-        assertEquals("ObjectName", directory.getObject(tags[11].getTagType()));
+        assertEquals("ObjectName", directory.getString(tags[11].getTagType()));
 
         assertEquals(IptcDirectory.TAG_SOURCE, tags[12].getTagType());
-        assertEquals("Source", directory.getObject(tags[12].getTagType()));
+        assertEquals("Source", directory.getString(tags[12].getTagType()));
 
         assertEquals(IptcDirectory.TAG_CREDIT, tags[13].getTagType());
-        assertEquals("Credits", directory.getObject(tags[13].getTagType()));
+        assertEquals("Credits", directory.getString(tags[13].getTagType()));
 
         assertEquals(IptcDirectory.TAG_BY_LINE_TITLE, tags[14].getTagType());
-        assertEquals("BylineTitle", directory.getObject(tags[14].getTagType()));
+        assertEquals("BylineTitle", directory.getString(tags[14].getTagType()));
 
         assertEquals(IptcDirectory.TAG_BY_LINE, tags[15].getTagType());
-        assertEquals("Byline", directory.getObject(tags[15].getTagType()));
+        assertEquals("Byline", directory.getString(tags[15].getTagType()));
     }
 
     @Test
@@ -122,52 +123,52 @@ public class IptcReaderTest
         assertEquals(2, directory.getObject(tags[0].getTagType()));
 
         assertEquals(IptcDirectory.TAG_CAPTION, tags[1].getTagType());
-        assertEquals("Caption PS6", directory.getObject(tags[1].getTagType()));
+        assertEquals("Caption PS6", directory.getString(tags[1].getTagType()));
 
         assertEquals(IptcDirectory.TAG_CAPTION_WRITER, tags[2].getTagType());
-        assertEquals("CaptionWriter", directory.getObject(tags[2].getTagType()));
+        assertEquals("CaptionWriter", directory.getString(tags[2].getTagType()));
 
         assertEquals(IptcDirectory.TAG_HEADLINE, tags[3].getTagType());
-        assertEquals("Headline", directory.getObject(tags[3].getTagType()));
+        assertEquals("Headline", directory.getString(tags[3].getTagType()));
 
         assertEquals(IptcDirectory.TAG_SPECIAL_INSTRUCTIONS, tags[4].getTagType());
-        assertEquals("Special Instr.", directory.getObject(tags[4].getTagType()));
+        assertEquals("Special Instr.", directory.getString(tags[4].getTagType()));
 
         assertEquals(IptcDirectory.TAG_BY_LINE, tags[5].getTagType());
-        assertEquals("Byline", directory.getObject(tags[5].getTagType()));
+        assertEquals("Byline", directory.getString(tags[5].getTagType()));
 
         assertEquals(IptcDirectory.TAG_BY_LINE_TITLE, tags[6].getTagType());
-        assertEquals("BylineTitle", directory.getObject(tags[6].getTagType()));
+        assertEquals("BylineTitle", directory.getString(tags[6].getTagType()));
 
         assertEquals(IptcDirectory.TAG_CREDIT, tags[7].getTagType());
-        assertEquals("Credits", directory.getObject(tags[7].getTagType()));
+        assertEquals("Credits", directory.getString(tags[7].getTagType()));
 
         assertEquals(IptcDirectory.TAG_SOURCE, tags[8].getTagType());
-        assertEquals("Source", directory.getObject(tags[8].getTagType()));
+        assertEquals("Source", directory.getString(tags[8].getTagType()));
 
         assertEquals(IptcDirectory.TAG_OBJECT_NAME, tags[9].getTagType());
-        assertEquals("ObjectName", directory.getObject(tags[9].getTagType()));
+        assertEquals("ObjectName", directory.getString(tags[9].getTagType()));
 
         assertEquals(IptcDirectory.TAG_CITY, tags[10].getTagType());
-        assertEquals("City", directory.getObject(tags[10].getTagType()));
+        assertEquals("City", directory.getString(tags[10].getTagType()));
 
         assertEquals(IptcDirectory.TAG_PROVINCE_OR_STATE, tags[11].getTagType());
-        assertEquals("State", directory.getObject(tags[11].getTagType()));
+        assertEquals("State", directory.getString(tags[11].getTagType()));
 
         assertEquals(IptcDirectory.TAG_COUNTRY_OR_PRIMARY_LOCATION_NAME, tags[12].getTagType());
-        assertEquals("Country", directory.getObject(tags[12].getTagType()));
+        assertEquals("Country", directory.getString(tags[12].getTagType()));
 
         assertEquals(IptcDirectory.TAG_ORIGINAL_TRANSMISSION_REFERENCE, tags[13].getTagType());
-        assertEquals("Transmission", directory.getObject(tags[13].getTagType()));
+        assertEquals("Transmission", directory.getString(tags[13].getTagType()));
 
         assertEquals(IptcDirectory.TAG_CATEGORY, tags[14].getTagType());
-        assertEquals("Cat", directory.getObject(tags[14].getTagType()));
+        assertEquals("Cat", directory.getString(tags[14].getTagType()));
 
         assertEquals(IptcDirectory.TAG_SUPPLEMENTAL_CATEGORIES, tags[15].getTagType());
         assertArrayEquals(new String[] { "Supl. Category1", "Supl. Category2" }, directory.getStringArray(tags[15].getTagType()));
 
         assertEquals(IptcDirectory.TAG_COPYRIGHT_NOTICE, tags[16].getTagType());
-        assertEquals("Copyright", directory.getObject(tags[16].getTagType()));
+        assertEquals("Copyright", directory.getString(tags[16].getTagType()));
     }
 
     @Test
@@ -190,7 +191,7 @@ public class IptcReaderTest
         assertEquals(2, directory.getObject(tags[2].getTagType()));
 
         assertEquals(IptcDirectory.TAG_CAPTION, tags[3].getTagType());
-        assertEquals("In diesem Text sind Umlaute enthalten, nämlich öfter als üblich: ÄÖÜäöüß\r", directory.getObject(tags[3].getTagType()));
+        assertEquals("In diesem Text sind Umlaute enthalten, nämlich öfter als üblich: ÄÖÜäöüß\r", directory.getStringValue(tags[3].getTagType()).toString());
     }
 
     @Test
@@ -210,7 +211,7 @@ public class IptcReaderTest
         assertEquals(2, directory.getObject(tags[1].getTagType()));
 
         assertEquals(IptcDirectory.TAG_CAPTION, tags[2].getTagType());
-        assertEquals("In diesem Text sind Umlaute enthalten, nämlich öfter als üblich: ÄÖÜäöüß\r", directory.getObject(tags[2].getTagType()));
+        assertEquals("In diesem Text sind Umlaute enthalten, nämlich öfter als üblich: ÄÖÜäöüß\r", directory.getStringValue(tags[2].getTagType()).toString());
     }
 
     @Test
@@ -227,7 +228,7 @@ public class IptcReaderTest
         assertEquals(2, directory.getObject(tags[0].getTagType()));
 
         assertEquals(IptcDirectory.TAG_CAPTION, tags[1].getTagType());
-        assertEquals("Das Encoding dieser Metadaten ist nicht deklariert und lässt sich nur schwer erkennen.", directory.getObject(tags[1].getTagType()));
+        assertEquals("Das Encoding dieser Metadaten ist nicht deklariert und lässt sich nur schwer erkennen.", directory.getStringValue(tags[1].getTagType()).toString());
 
         assertEquals(IptcDirectory.TAG_KEYWORDS, tags[2].getTagType());
         assertArrayEquals(new String[]{"häufig", "üblich", "Lösung", "Spaß"}, directory.getStringArray(tags[2].getTagType()));
