@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2015 Drew Noakes
+ * Copyright 2002-2017 Drew Noakes
  *
  *    Licensed under the Apache License, Version 2.0 (the "License");
  *    you may not use this file except in compliance with the License.
@@ -49,12 +49,12 @@ public class JfifReaderTest
         reader.extract(new ByteArrayReader(jfifData), metadata);
 
         assertEquals(1, metadata.getDirectoryCount());
-        JfifDirectory directory = metadata.getDirectory(JfifDirectory.class);
+        JfifDirectory directory = metadata.getFirstDirectoryOfType(JfifDirectory.class);
         assertNotNull(directory);
         assertFalse(directory.getErrors().toString(), directory.hasErrors());
 
         Tag[] tags = directory.getTags().toArray(new Tag[directory.getTagCount()]);
-        assertEquals(4, tags.length);
+        assertEquals(6, tags.length);
 
         assertEquals(JfifDirectory.TAG_VERSION, tags[0].getTagType());
         assertEquals(0x0102, directory.getInt(tags[0].getTagType()));
@@ -67,5 +67,11 @@ public class JfifReaderTest
 
         assertEquals(JfifDirectory.TAG_RESY, tags[3].getTagType());
         assertEquals(108, directory.getInt(tags[3].getTagType()));
+
+        assertEquals(JfifDirectory.TAG_THUMB_WIDTH, tags[4].getTagType());
+        assertEquals(0, directory.getInt(tags[4].getTagType()));
+
+        assertEquals(JfifDirectory.TAG_THUMB_HEIGHT, tags[5].getTagType());
+        assertEquals(0, directory.getInt(tags[5].getTagType()));
     }
 }
